package client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import server.LoginException;
import server.UnnavailableLoginException;
import server.User;

import net.AppSocket;
import net.ConnectionInfo;
import net.TimeoutLimitException;
import net.Transport;

public class ServerConnection {
	private AppSocket sock;
	private final int TIMEOUT_TIME = 800;
	private ConnectionInfo sv;
	public ServerConnection(AppSocket sock, ConnectionInfo sv){
		this.sock = sock;
		this.sv = sv;
	}
	public void logar(String login, String senha) throws TimeoutLimitException, LoginException{
		ByteBuffer bf = ByteBuffer.allocate(login.length()+senha.length()+3);
		bf.put((byte)1);
		bf.put((byte)login.length());
		bf.put(login.getBytes());
		bf.put((byte)senha.length());
		bf.put(senha.getBytes());
		bf.flip();
		sock.send(bf.array());
		byte[] resposta = sock.receive(TIMEOUT_TIME);
		if(resposta != null){
			if(resposta[0] == 3){
				throw new LoginException();
			}
		}else{
			throw new TimeoutLimitException();
		}
	}
	public void connect() throws TimeoutLimitException{
		sock.connect(sv);
	}
	public void addFriend(String login) throws LoginException, ServerException, TimeoutLimitException{
		ByteBuffer bf = ByteBuffer.allocate(login.length()+2);
		bf.put((byte)5);
		bf.put((byte)login.length());
		bf.put(login.getBytes());
		bf.flip();
		sock.send(bf.array());
		byte[] resposta = sock.receive(TIMEOUT_TIME);
		if(resposta != null){
			if(resposta[0] == 3){
				throw new LoginException();
			}else if(resposta[0] == 1){
				throw new ServerException();
			}
		}else{
			throw new TimeoutLimitException();
		}
	}
	public void cadastrar(User usuario) throws IOException, TimeoutLimitException, UnnavailableLoginException, ServerException{
		 ByteArrayOutputStream out = new ByteArrayOutputStream();
		 ObjectOutputStream objOut = new ObjectOutputStream(out);
		 objOut.writeObject(usuario);
		 ByteBuffer bf = ByteBuffer.allocate(out.size()+1);
		 out.flush();
		 bf.put((byte)3);
		 bf.put(out.toByteArray());
		 bf.flip();
		 sock.send(bf.array());
		 byte[] resposta = sock.receive(TIMEOUT_TIME);
		if(resposta != null){
			 if(resposta[0] == 2){
				 throw new UnnavailableLoginException();
			 }else if(resposta[0] == 1){
				 throw new ServerException();
			 }
		}else{
			throw new TimeoutLimitException();
		}
	}
	@SuppressWarnings("unchecked")
	public ArrayList<String> getFriends() throws TimeoutLimitException, IOException, ClassNotFoundException, ServerException{
		byte[] req = {6};
		sock.send(req);
		byte[] resposta = sock.receive(TIMEOUT_TIME);
		ArrayList<String> retorno = null;
		if(resposta != null){
			if(resposta[0] != 1){
				ByteArrayInputStream  inputStream = new ByteArrayInputStream(resposta);
				ObjectInputStream objStream = new ObjectInputStream(inputStream);
				Object temp = objStream.readObject();
				
				if(temp instanceof ArrayList<?>){
					retorno = (ArrayList<String>)temp;
				}
			}else{
				throw new ServerException();
			}
		}
		return retorno;
	}
	public void logout() throws TimeoutLimitException{
		byte[] logoutRequest = {2};
		sock.send(logoutRequest);
		
	}
	@SuppressWarnings("unchecked")
	public Hashtable<String, ConnectionInfo> getOnlineFriends() throws TimeoutLimitException, IOException, ClassNotFoundException, ServerException{
		byte[] req = { 4 };
		sock.send(req);
		byte[] resposta = sock.receive(TIMEOUT_TIME);
		Hashtable<String, ConnectionInfo> retorno = null;
		if(resposta != null){

			if(resposta[0] != 1){
				ByteArrayInputStream  inputStream = new ByteArrayInputStream(resposta);
				ObjectInputStream objStream = new ObjectInputStream(inputStream);
				Object temp = objStream.readObject();
				
				if(temp instanceof Hashtable<? ,?>){
					retorno = (Hashtable<String, ConnectionInfo>)temp;
				}
				}else{
					throw new ServerException();
				}
		}
		return retorno;
	}
	public static void main(String args[]) throws SocketException, UnknownHostException, TimeoutLimitException, ServerException{
		Transport trans1 = new Transport(5000);
		ConnectionInfo ci = new ConnectionInfo(0,InetAddress.getByName("172.20.4.228"), 3000);
		AppSocket socket = new AppSocket(trans1, 3);
		ServerConnection teste = new ServerConnection(socket, ci);
		teste.connect();
		User usuario = new User( "igor.calabria", "123", "coisa@hotmail.com", "@malaaao", "coisa", "offline");
		User usuario2 = new User( "marcel.gay", "123", "coi2sa@hotmail.com", "@malaaao", "coisa", "offline");
		try {
			teste.logar("igor.calabria", "123");
			System.out.println(teste.getOnlineFriends());
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
