package server;

import net.AppSocket;
import net.ConnectionInfo;
import net.TimeoutLimitException;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;


public class ServerThread extends Thread{
	private AppSocket sock;
	private Server sv;
	private boolean vive;
	private ConnectionInfo destino;
	private String login;
	public ServerThread(Server sv, AppSocket sock, ConnectionInfo destino){
		this.sv = sv;
		this.sock = sock;
		this.destino = destino;
		this.login = "";
		vive = true;
	}
	/*Pacotes de resposta:
	 *  0 = ok
	 *  1 = erroInterno
	 *  2 = login j existente
	 * 	3 = erro de login,
	 * 
	 * @see java.lang.Thread#run()
	 */
	public void run(){
		while(vive){
			byte[] req = sock.receive(45000);
			if (req != null) {
				switch (req[0]) {
				case 1: // Logar
					System.out.println("Server: Recebeu requisio para Logar do ip: "+destino.getIp().getHostAddress());
					String login = "";
					String senha = "";
					boolean erro = false;
					byte tamanhoLogin = req[1];
					for (int i = 0; i < tamanhoLogin; i++) {
						login += (char) req[2 + i];
					}
					for (int i = 0; i < req[2 + req[1]]; i++) {
						senha += (char) req[3 + req[1] + i];
					}
					try {
						sv.login(login, senha, destino);
					} catch (LoginException e2) {
						erro = true;
					} catch (UsuarioNaoExistenteException e2) {
						erro = true;
					}
					try {
						if (erro) {
							sock.send(new byte[] { 3 });
						} else {
							sock.send(new byte[] { 0 });
						}
						this.login = login;
					} catch (TimeoutLimitException e) {
						this.kill();
					}

					break;
				case 2: // Logout
					System.out.println("Server: Recebeu requisio para Deslogar do ip: "+destino.getIp().getHostAddress());
					try {
						sv.logout(this.login);
						
					} catch (LoginException e1) {

					} catch (UsuarioNaoExistenteException e1) {

					}
					
					this.kill();
					break;
				case 3: // Cadastro
					System.out.println("Server: Recebeu requisio para cadastrar do ip: "+destino.getIp().getHostAddress());
					User usuario = null;
					boolean loginExistente = false;
					boolean erroGeral = false;
					try {
						usuario = this.getUserFromStream(req);
					} catch (IOException e1) {
						erroGeral = true;
					} catch (ClassNotFoundException e1) {
						erroGeral = true;
					}
				
					if(usuario != null){
						try {
							sv.addUser(usuario.getLogin(), usuario.getSenha(), usuario.getEmail(), usuario.getTwitter());
						} catch (UnnavailableLoginException e) {
							loginExistente = true;
						} catch (IOException e) {
							erroGeral = true;
						}
					}
					if(erroGeral){
						try {
							sock.send(new byte[]{1});
						} catch (TimeoutLimitException e) {
							this.kill();
						}
					}else if(loginExistente){
			
						try{
							sock.send(new byte[]{2});
						}catch(TimeoutLimitException e){
							this.kill();
						}
					}else{
						try {
							sock.send(new byte[]{0});
						} catch (TimeoutLimitException e) {
							this.kill();
						}
					}
					break;
				case 4: // Pede as informaoes dos amigos
					System.out.println("Server: Recebeu requisio para mandar lista Online do ip: "+destino.getIp().getHostAddress());
					ConcurrentHashMap<String, ConnectionInfo> listaOnline = sv
							.getListaOnline();
					System.out.println(listaOnline.toString());
					boolean erro4 = false;
					if (this.login != "") {
						Hashtable<String, ConnectionInfo> retorno = new Hashtable<String, ConnectionInfo>();
						try {
							ArrayList<String> amigos = sv.getContatos(this.login);
							for (String amigo : amigos) {
								if (listaOnline.containsKey(amigo)) {
									retorno.put(amigo, listaOnline
											.get(amigo));
								}
							}
						} catch (UsuarioNaoExistenteException e) {
							erro4 = true;
						} catch (IOException e) {
							erro4 = true;
						}
						if (!erro4) {
							ByteArrayOutputStream stream = new ByteArrayOutputStream();
							try {
								ObjectOutputStream objOut = new ObjectOutputStream(
										stream);
								objOut.writeObject(retorno);
								stream.flush();
								byte[] manda = stream.toByteArray();
								sock.send(manda);
							} catch (IOException e) {
								e.printStackTrace();
							} catch (TimeoutLimitException e) {
								this.kill();
							}
						} else {
							try {
								sock.send(new byte[] { 1 });
							} catch (TimeoutLimitException e) {
								this.kill();
							}
						}
					}
					break;
				case -33:
					break;
				case 6:// retorna lista de amigos
					System.out.println("Server: Recebeu requisio para mandar lista de amigos do ip: "+destino.getIp().getHostAddress());
					boolean erroAmigos = false;
					try {
						ArrayList<String> amigos = sv.getContatos(this.login);
						ByteArrayOutputStream stream = new ByteArrayOutputStream();
						ObjectOutputStream objOut = new ObjectOutputStream(
								stream);
						objOut.writeObject(amigos);
						stream.flush();
						byte[] manda = stream.toByteArray();
						sock.send(manda);
					} catch (UsuarioNaoExistenteException e1) {
						erroAmigos = true;
					} catch (IOException e1) {
						erroAmigos = true;
					} catch (TimeoutLimitException e) {
						erroAmigos = true;
						this.kill();
					}
					try{
						if(erroAmigos){
							sock.send(new byte[]{1});
						}
					}catch(TimeoutLimitException e){
						this.kill();
					}
					break;
				case 5: // Adiciona amigo
					System.out.println("Server: Recebeu requisio para adicionar amigo do ip: "+destino.getIp().getHostAddress());
					String amigo = "";
					boolean naoAchou = false;
					boolean ioError = false;
					for (int i = 0; i < req[1]; i++) {
						amigo += (char) req[2 + i];
					}
					try {
						sv.addFriend(this.login, amigo);
					} catch (UsuarioNaoExistenteException e) {
						naoAchou = true;
					} catch (IOException e) {
						ioError = true;
					}
					try {
						if (naoAchou) {
							sock.send(new byte[] { 3 });
						} else if (ioError) {
							sock.send(new byte[] { 1 });
						} else {
							sock.send(new byte[] { 0 });
						}
					} catch (TimeoutLimitException e) {
						this.kill();
					}
				}

			}else{
				this.kill();
			}
		}	
	}
	private User getUserFromStream(byte[] request) throws IOException, ClassNotFoundException{
		User retorno = null;
		ByteArrayInputStream inputStream = new ByteArrayInputStream(
				request, 1, request.length - 1);
		ObjectInputStream objInput = null;
		objInput = new ObjectInputStream(inputStream);
	
		if (objInput != null) {
			Object temp = objInput.readObject();
			if(temp instanceof User){
				retorno = (User)temp;
			}
		}
		return retorno;
	}
	public void kill(){
		vive = false;
		try {
			sv.logout(login);
		} catch (LoginException e) {

		} catch (UsuarioNaoExistenteException e) {
	
		}
		sv.getGuiListener().userDisconneected(destino.getIp().getHostAddress());
		sv.getPeers().remove(destino);
		sock.kill();
		System.out.println("morreu Thread: "+sock.getIdSocket());
	}
}
